@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: response_editor.c,v 1.12 2004/04/06 18:12:17 fuyu Exp $
+ * $Id: response_editor.c,v 1.12.2.2 2004/09/24 16:13:18 fuyu Exp $
  */
 
 #include "config.h"
@@ -125,6 +125,21 @@ response_editor_class_init(ResponseEditorClass *klass)
 }
 
 
+static void
+response_editor_insert_aa_button_cb(gpointer unused, ResponseEditor *editor)
+{
+  gchar *aa = response_editor_get_current_aa(editor);
+
+  if (aa == NULL)
+    return;
+
+  response_editor_insert_response(editor, aa);
+  g_free(aa);
+
+  return;
+}
+
+
 static void
 response_editor_init(ResponseEditor *editor)
 {
@@ -135,6 +150,11 @@ response_editor_init(ResponseEditor *editor)
   GtkWidget *mail_combo;
   GtkWidget *scrolled_window;
   GtkWidget *text_view;
+#if GTK_MINOR_VERSION > 2
+  GtkWidget *aa_hbox;
+  GtkWidget *aa_combo;
+  GtkWidget *aa_button;
+#endif
 
   /* hboxν */
   hbox = gtk_hbox_new(FALSE, 5);
@@ -183,6 +203,25 @@ response_editor_init(ResponseEditor *editor)
   gtk_text_view_set_wrap_mode(editor->text_view, GTK_WRAP_CHAR);
   gtk_widget_show(text_view);
   gtk_container_add(GTK_CONTAINER(scrolled_window), text_view);
+
+#if GTK_MINOR_VERSION > 2
+  aa_hbox = gtk_hbox_new(FALSE, 5);
+  editor->aa_hbox = GTK_BOX(aa_hbox);
+  gtk_widget_show(aa_hbox);
+  gtk_box_pack_start(GTK_BOX(&editor->container), aa_hbox, FALSE, FALSE, 0);
+
+  aa_combo = gtk_combo_box_new_text();
+  editor->aa_combo = GTK_COMBO_BOX(aa_combo);
+  gtk_widget_show(aa_combo);
+  gtk_box_pack_start(GTK_BOX(aa_hbox), aa_combo, TRUE, TRUE, 0);
+
+  aa_button = gtk_button_new_with_label(_("Insert"));
+  g_signal_connect(G_OBJECT(aa_button), "clicked",
+		   G_CALLBACK(response_editor_insert_aa_button_cb), editor);
+  gtk_widget_show(aa_button);
+
+  gtk_box_pack_start(GTK_BOX(aa_hbox), aa_button, FALSE, FALSE, 0);
+#endif
 }
 
 
@@ -355,6 +394,36 @@ response_editor_get_response(ResponseEditor *editor)
 }
 
 
+void
+response_editor_append_response(ResponseEditor *editor, const gchar *response)
+{
+  GtkTextIter end_iter;
+  GtkTextBuffer *buffer;
+
+  g_return_if_fail(IS_RESPONSE_EDITOR(editor));
+  g_return_if_fail(response != NULL);
+
+  buffer = gtk_text_view_get_buffer(editor->text_view);
+  gtk_text_buffer_get_end_iter(buffer, &end_iter);
+  if (gtk_text_buffer_get_char_count(buffer) > 0)
+    gtk_text_buffer_insert(buffer, &end_iter, "\n", 1);
+  gtk_text_buffer_insert(buffer, &end_iter, response, -1);
+}
+
+
+void
+response_editor_insert_response(ResponseEditor *editor, const gchar *response)
+{
+  GtkTextBuffer *buffer;
+
+  g_return_if_fail(IS_RESPONSE_EDITOR(editor));
+  g_return_if_fail(response != NULL);
+
+  buffer = gtk_text_view_get_buffer(editor->text_view);
+  gtk_text_buffer_insert_interactive_at_cursor(buffer, response, -1, TRUE);
+}
+
+
 #if 0
 void
 response_editor_set_response(ResponseEditor *editor, const gchar *response)
@@ -362,3 +431,35 @@ response_editor_set_response(ResponseEditor *editor, const gchar *response)
   g_return_if_fail(IS_RESPONSE_EDITOR(editor));
 }
 #endif
+
+
+#if GTK_MINOR_VERSION > 2
+void
+response_editor_add_aa(ResponseEditor *editor, const gchar *aa)
+{
+  g_return_if_fail(IS_RESPONSE_EDITOR(editor));
+
+  if (aa == NULL)
+    return;
+
+  gtk_combo_box_append_text(editor->aa_combo, aa);
+}
+
+
+gchar *
+response_editor_get_current_aa(ResponseEditor *editor)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gchar *aa = NULL;
+
+  g_return_val_if_fail(IS_RESPONSE_EDITOR(editor), NULL);
+
+  model = gtk_combo_box_get_model(editor->aa_combo);
+  if (!gtk_combo_box_get_active_iter(editor->aa_combo, &iter))
+    return NULL;
+
+  gtk_tree_model_get(model, &iter, 0, &aa, -1);
+  return aa;
+}
+#endif
