@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.27 2004/04/13 15:38:05 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.27.2.7 2004/06/08 15:32:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -196,6 +196,9 @@ ochusha_bbs_table_init(OchushaBBSTable *table)
   table->board_name_table
     = g_hash_table_new_full(g_str_hash, g_str_equal, NULL,
 			    (GDestroyNotify)OCHU_OBJECT_UNREF_FUNC);
+  table->board_id_table_2ch
+    = g_hash_table_new_full(g_str_hash, g_str_equal, NULL,
+			    (GDestroyNotify)OCHU_OBJECT_UNREF_FUNC);
 }
 
 
@@ -231,6 +234,12 @@ ochusha_bbs_table_finalize(GObject *object)
       table->board_name_table = NULL;
     }
 
+  if (table->board_id_table_2ch != NULL)
+    {
+      g_hash_table_destroy(table->board_id_table_2ch);
+      table->board_id_table_2ch = NULL;
+    }
+
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
@@ -306,12 +315,35 @@ void
 ochusha_bbs_table_add_board(OchushaBBSTable *table,
 			    OchushaBulletinBoard *board)
 {
+  char past_url[PATH_MAX];
   g_return_if_fail(OCHUSHA_IS_BBS_TABLE(table)
 		   && OCHUSHA_IS_BULLETIN_BOARD(board));
 
   OCHU_OBJECT_REF(board);
   g_hash_table_insert(table->board_table, board->base_url, board);
 
+  if (OCHUSHA_IS_BOARD_2CH(board))
+    {
+      OchushaBoard2ch *board_2ch = OCHUSHA_BOARD_2CH(board);
+      GSList *entry = board_2ch->server_history;
+      while (entry != NULL)
+	{
+	  snprintf(past_url, PATH_MAX, "http://%s%s%s/",
+		   (const char *)entry->data, board->base_path, board->id);
+	  OCHU_OBJECT_REF(board);
+	  g_hash_table_insert(table->board_table, past_url, board);
+	  entry = entry->next;
+	}
+      OCHU_OBJECT_REF(board);
+      if (board->bbs_type == OCHUSHA_BBS_TYPE_2CH
+	  || board->bbs_type == OCHUSHA_BBS_TYPE_MACHIBBS)
+	g_hash_table_insert(table->board_id_table_2ch, board->id, board);
+#if 0
+      fprintf(stderr, "Insert board(URL=%s, id=%s) into board_id_table_2ch\n",
+	      board->base_url, board->id);
+#endif
+    }
+
   OCHU_OBJECT_REF(board);
   g_hash_table_insert(table->board_name_table, board->name, board);
 
@@ -343,9 +375,29 @@ ochusha_bbs_table_remove_board(OchushaBBSTable *table,
 OchushaBulletinBoard *
 ochusha_bbs_table_lookup_board_by_url(OchushaBBSTable *table, const char *url)
 {
-  g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && url != NULL, NULL);
+  OchushaBulletinBoard *board;
+  char *board_url = NULL;
+  int bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+  char *board_id = NULL;
 
-  return g_hash_table_lookup(table->board_table, url);
+  g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && url != NULL, NULL);
+  board = g_hash_table_lookup(table->board_table, url);
+  if (board != NULL)
+    return board;
+
+  if (ochusha_utils_2ch_check_url(url, &board_url, &bbs_type,
+				  &board_id, NULL, NULL, NULL, NULL))
+    {
+      g_return_val_if_fail(board_id != NULL, NULL);
+      if (strstr(board_url, ".2ch.net/") != NULL
+	  || strstr(board_url, ".bbspink.com/") != NULL)
+	board = g_hash_table_lookup(table->board_id_table_2ch, board_id);
+      if (board_url != NULL)
+	G_FREE(board_url);
+      if (board_id != NULL)
+	G_FREE(board_id);
+    }
+  return board;
 }
 
 
@@ -393,24 +445,41 @@ ochusha_bbs_table_check_url(OchushaBBSTable *table, const char *url,
 {
   char *board_url = NULL;
   gboolean result;
+  int bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+  char *board_id = NULL;
 
-  if ((result = ochusha_utils_2ch_check_url(url, &board_url, bbs_type_p,
-					    NULL, thread_id_p, from_p, to_p,
-					    is_kako_html_p)))
+  if ((result = ochusha_utils_2ch_check_url(url, &board_url, &bbs_type,
+					    &board_id, thread_id_p,
+					    from_p, to_p, is_kako_html_p)))
     {
-      if (board_p != NULL)
+#if 0
+      fprintf(stderr, "ochusha_bbs_table_check_url: %s, %s, %d, %s\n",
+	      url, board_url, bbs_type, board_id);
+#endif
+      if (board_p != NULL && board_id != NULL)
 	{
 	  *board_p = ochusha_bbs_table_lookup_board_by_url(table, board_url);
+	  if (*board_p == NULL && bbs_type == OCHUSHA_BBS_TYPE_2CH
+	      && (strstr(board_url, "2ch.net/") != NULL
+		  || strstr(board_url, "bbspink.com/") != NULL))
+	    *board_p = g_hash_table_lookup(table->board_id_table_2ch,
+					   board_id);
+
 	  if (bbs_type_p != NULL)
 	    {
 	      if (*board_p != NULL)
 		*bbs_type_p = (*board_p)->bbs_type;
-	      else if (*bbs_type_p == OCHUSHA_BBS_TYPE_2CH)
+	      else if (bbs_type == OCHUSHA_BBS_TYPE_2CH)
 		*bbs_type_p = OCHUSHA_BBS_TYPE_2CH_COMPATIBLE;
 	    }
 	}
+      else if (bbs_type_p != NULL)
+	*bbs_type_p = bbs_type;
     }
 
+  if (board_id != NULL)
+    G_FREE(board_id);
+
   if (board_url_p != NULL)
     *board_url_p = board_url;
   else if (board_url != NULL)
@@ -880,36 +949,55 @@ endElementHandler(void *context, const xmlChar *name)
 						sax_context->board_attributes,
 						"bbs_type");
 
-	      g_signal_emit(G_OBJECT(table),
-			    bbs_table_signals[BOARD_NEW_SIGNAL],
-			    0, bbs_type, board_name, base_url, &board);
-
-	      if (board == NULL)
+	      if (strstr(base_url, "info.2ch.net") == NULL)
 		{
-		  switch (bbs_type)
+		  g_signal_emit(G_OBJECT(table),
+				bbs_table_signals[BOARD_NEW_SIGNAL],
+				0, bbs_type, board_name, base_url, &board);
+
+		  if (board == NULL)
 		    {
-		    case OCHUSHA_BBS_TYPE_2CH:
-		    case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
-		    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
-		    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-		      board = ochusha_board_2ch_new(board_name, base_url);
-		      break;
+#if 1
+		      /* ߤμǤϡĤͤȤޤBBSξ
+		       * ˤ뤻ǡİžνäƤ롣0.5ϤǤ
+		       * ޤȤк򤹤ĤϤʤΤǡad-hoc
+		       * бƤ
+		       */
+		      if (bbs_type == OCHUSHA_BBS_TYPE_2CH
+			  && strstr(base_url, "machi.to") != NULL)
+			bbs_type = OCHUSHA_BBS_TYPE_MACHIBBS;
+		      else if (bbs_type == OCHUSHA_BBS_TYPE_MACHIBBS
+			       && strstr(base_url, "2ch.net") != NULL)
+			bbs_type = OCHUSHA_BBS_TYPE_2CH;
+#endif
+		      switch (bbs_type)
+			{
+			case OCHUSHA_BBS_TYPE_2CH:
+			case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+			case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
+			case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
+			  board = ochusha_board_2ch_new(board_name, base_url);
+			  break;
 
-		    case OCHUSHA_BBS_TYPE_JBBS:
-		    case OCHUSHA_BBS_TYPE_MACHIBBS:
-		    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-		    case OCHUSHA_BBS_TYPE_MITINOKU:
-		      board = ochusha_board_jbbs_new(board_name, base_url);
-		      break;
+			case OCHUSHA_BBS_TYPE_JBBS:
+			case OCHUSHA_BBS_TYPE_MACHIBBS:
+			case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
+			case OCHUSHA_BBS_TYPE_MITINOKU:
+			  board = ochusha_board_jbbs_new(board_name, base_url);
+			  break;
 
-		    default:
-		      /* board = NULL;  XXX ȤꤢĤ̵ */
-		      break;
+			default:
+			  /* board = NULL;  XXX ȤꤢĤ̵ */
+			  break;
+			}
 		    }
 		}
 
 	      if (board != NULL)
 		{
+		  if (bbs_type == OCHUSHA_BBS_TYPE_2CH
+		      && strstr(base_url, "headline.2ch.net") != NULL)
+		    bbs_type = OCHUSHA_BBS_TYPE_2CH_HEADLINE;
 		  ochusha_bulletin_board_set_bbs_type(board, bbs_type);
 #if DEBUG_SAX_HANDLER
 		  fprintf(stderr, "adding board: %s\n", board_name);
