@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.45.2.3 2004/06/16 05:24:36 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.45.2.4 2004/09/22 14:31:28 fuyu Exp $
  */
 
 #include "config.h"
@@ -485,6 +485,30 @@ static char *
 ochusha_board_2ch_generate_base_path(OchushaBulletinBoard *board,
 				     const char *url)
 {
+  char *path;
+  char *tmp_pos;
+  char *abs_path = ochusha_utils_url_extract_http_absolute_path(url);
+  size_t len;
+
+  g_return_val_if_fail(abs_path != NULL, G_STRDUP("/"));
+  len = strlen(abs_path);
+  if (len < 1 || abs_path[0] != '/')
+    goto error;
+
+  if (abs_path[len - 1] == '/')
+    abs_path[len - 1] = '\0';
+
+  tmp_pos = strrchr(abs_path + 1, '/');
+  if (tmp_pos == NULL)
+    goto error;
+
+  tmp_pos[1] = '\0';
+  path = G_STRDUP(abs_path);
+  G_FREE(abs_path);
+  return path;
+
+ error:
+  G_FREE(abs_path);
   return G_STRDUP("/");
 }
 
@@ -494,38 +518,29 @@ ochusha_board_2ch_generate_board_id(OchushaBulletinBoard *board,
 				    const char *url)
 {
   char *id;
+  char *tmp_pos;
   char *abs_path = ochusha_utils_url_extract_http_absolute_path(url);
-  if (abs_path != NULL && abs_path[0] != '/')
-    {
-      G_FREE(abs_path);
-      abs_path = NULL;
-    }
+  size_t len;
   g_return_val_if_fail(abs_path != NULL, NULL);
 
-  switch (board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
-    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
-    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-      {
-	char *tmp_pos = strchr(abs_path + 1, '/');
-	if (tmp_pos != NULL)
-	  {
-	    *tmp_pos = '\0';
-	  }
-	id = G_STRDUP(abs_path + 1);
-	break;
-      }
+  len = strlen(abs_path);
+  if (len < 1 || abs_path[0] != '/')
+    goto error;
 
-    default:
-      id = NULL;	/* ̤ݡ */
-      break;
-    }
+  if (abs_path[len - 1] == '/')
+    abs_path[len - 1] = '\0';
 
+  tmp_pos = strrchr(abs_path, '/');
+  if (tmp_pos == NULL)
+    goto error;
+
+  id = G_STRDUP(tmp_pos + 1);
   G_FREE(abs_path);
-
   return id;
+
+ error:
+  G_FREE(abs_path);
+  return NULL;
 }
 
 
@@ -712,10 +727,10 @@ ochusha_board_2ch_is_new_thread_supported(OchushaBulletinBoard *board)
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
+    case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
       return TRUE;
 
-    case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
     default:
       return FALSE;
     }
