@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.60.2.9 2004/06/26 18:02:59 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.60.2.10 2004/11/09 22:43:06 fuyu Exp $
  */
 
 #include "config.h"
@@ -1969,7 +1969,7 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 static void
 background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
-  int fd = (int)g_object_get_qdata(G_OBJECT(buffer), filedesc_id);
+  int fd = (int)(long)g_object_get_qdata(G_OBJECT(buffer), filedesc_id);
   OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
 						    broker_id);
   OchushaNetworkBrokerBufferStatus *status
@@ -2134,7 +2134,8 @@ ochusha_network_broker_read_from_url_full(OchushaNetworkBroker *broker,
 	  if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
 	    buffer = ochusha_async_buffer_new(NULL, 0, NULL);
 
-	  g_object_set_qdata(G_OBJECT(buffer), filedesc_id, (gpointer)fd);
+	  g_object_set_qdata(G_OBJECT(buffer), filedesc_id,
+			     (gpointer)(long)fd);
 	  g_object_set_qdata(G_OBJECT(buffer), broker_id, broker);
 	  g_object_set_qdata_full(G_OBJECT(buffer), broker_buffer_status_id,
 		status,
