@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: worker.h,v 1.3 2004/01/21 08:41:01 fuyu Exp $
+ * $Id: worker.h,v 1.4.2.1 2004/06/26 18:02:59 fuyu Exp $
  */
 
 #ifndef _WORKER_H_
@@ -58,10 +58,14 @@ typedef struct _WorkerJob
 
 
 void initialize_worker(int initial_num_threads, int maximum_num_threads,
-		       int num_modest_threads);
+		       int num_modest_threads, int num_loader_threads);
 
 void commit_job(WorkerJob *job);
 void commit_modest_job(WorkerJob *job);
+void commit_loader_job(WorkerJob *job);
 void terminate_workers(void);
+void join_workers(void);
+void cancel_pending_jobs(void);
+
 
 #endif	/* _WORKER_H_ */
