@@ -7,6 +7,17 @@
 **********************************************************************/
 #include "regenc.h"
 
+#define USE_INVALID_CODE_SCHEME
+
+#ifdef USE_INVALID_CODE_SCHEME
+/* virtual codepoint values for invalid encoding byte 0xfe and 0xff */
+#define INVALID_CODE_FE   0xfffffffe
+#define INVALID_CODE_FF   0xffffffff
+
+#define VALID_CODE_LIMIT  0x7fffffff
+#endif
+
+
 #define utf8_islead(c)     ((UChar )((c) & 0xc0) != 0x80)
 
 #define ENC_IS_ISO_8859_1_CTYPE(code,ctype) \
@@ -64,8 +75,14 @@ utf8_mbc_to_code(UChar* p, UChar* end)
     }
     return n;
   }
-  else
+  else {
+#ifdef USE_INVALID_CODE_SCHEME
+    if (c > 0xfd) {
+      return ((c == 0xfe) ? INVALID_CODE_FE : INVALID_CODE_FF);
+    }
+#endif
     return (OnigCodePoint )c;
+  }
 }
 
 static int
@@ -81,6 +98,10 @@ utf8_code_to_mbclen(OnigCodePoint code)
   else if ((code & 0xffe00000) == 0) return 4;
   else if ((code & 0xfc000000) == 0) return 5;
   else if ((code & 0x80000000) == 0) return 6;
+#ifdef USE_INVALID_CODE_SCHEME
+  else if (code == INVALID_CODE_FE) return 1;
+  else if (code == INVALID_CODE_FF) return 1;
+#endif
   else
     return ONIGENCERR_TOO_BIG_WIDE_CHAR_VALUE;
 }
@@ -147,6 +168,16 @@ utf8_code_to_mbc(OnigCodePoint code, UChar *buf)
       *p++ = UTF8_TRAILS(code, 12);
       *p++ = UTF8_TRAILS(code,  6);
     }
+#ifdef USE_INVALID_CODE_SCHEME
+    else if (code == INVALID_CODE_FE) {
+      *p = 0xfe;
+      return 1;
+    }
+    else if (code == INVALID_CODE_FF) {
+      *p = 0xff;
+      return 1;
+    }
+#endif
     else {
       return ONIGENCERR_TOO_BIG_WIDE_CHAR_VALUE;
     }
@@ -199,7 +230,10 @@ utf8_code_is_ctype(OnigCodePoint code, unsigned int ctype)
   }
 
   if ((ctype & ONIGENC_CTYPE_WORD) != 0) {
-    return TRUE;
+#ifdef USE_INVALID_CODE_SCHEME
+    if (code <= VALID_CODE_LIMIT)
+#endif
+      return TRUE;
   }
 
   return FALSE;
