@@ -57,6 +57,7 @@
 #define THREAD_ATOMIC_START         /* depend on thread system */
 #define THREAD_ATOMIC_END           /* depend on thread system */
 #define THREAD_PASS                 /* depend on thread system */
+#define CHECK_INTERRUPT             /* depend on application */
 #define xmalloc     malloc
 #define xrealloc    realloc
 #define xfree       free
@@ -67,6 +68,14 @@
 #define THREAD_ATOMIC_START          DEFER_INTS
 #define THREAD_ATOMIC_END            ENABLE_INTS
 #define THREAD_PASS                  rb_thread_schedule()
+#define CHECK_INTERRUPT do {\
+  if (rb_trap_pending) {\
+    if (! rb_prohibit_interrupt) {\
+      rb_trap_exec();\
+    }\
+  }\
+} while (0)
+
 #define DEFAULT_WARN_FUNCTION        rb_warn
 #define DEFAULT_VERB_WARN_FUNCTION   rb_warning
 
@@ -500,6 +509,8 @@ enum OpCode {
   OP_REPEAT_NG,            /* {n,m}? (non greedy) */
   OP_REPEAT_INC,
   OP_REPEAT_INC_NG,        /* non greedy */
+  OP_REPEAT_INC_SG,        /* search and get in stack */
+  OP_REPEAT_INC_NG_SG,     /* search and get in stack (non greedy) */
   OP_NULL_CHECK_START,     /* null loop checker start */
   OP_NULL_CHECK_END,       /* null loop checker end   */
   OP_NULL_CHECK_END_MEMST, /* null loop checker end (with capture status) */
@@ -528,11 +539,11 @@ enum OpCode {
 #define ARG_MEMNUM       4
 #define ARG_OPTION       5
 
-typedef short int   RelAddrType;
-typedef short int   AbsAddrType;
-typedef short int   LengthType;
-typedef short int   MemNumType;
-typedef int         RepeatNumType;
+typedef int RelAddrType;
+typedef int AbsAddrType;
+typedef int LengthType;
+typedef int RepeatNumType;
+typedef short int MemNumType;
 
 #define SIZE_OPCODE        1
 #define SIZE_RELADDR       sizeof(RelAddrType)
